/** @file
 * @brief Long-running "soak" tests for Xapian.
 */
/* Copyright (C) 2010 Richard Boulton
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see
 * <https://www.gnu.org/licenses/>.
 */
#ifndef XAPIAN_INCLUDED_SOAKTEST_H
#define XAPIAN_INCLUDED_SOAKTEST_H

/// Initialise the random number generator with the seed.
extern unsigned int initrand();

/// Return a random integer in the range 0 to s-1.
extern unsigned int randint(unsigned int s);

#endif // XAPIAN_INCLUDED_SOAKTEST_H
